@@ -1,83 +1,81 @@
-#line 1
-# 8667e12ea286715fb3e93c82b3356305890112f1
+#!/usr/bin/perl
 
 package Crypt::Util;
+use Moose;
 
-use strict;
-use warnings;
-
-use base qw/Class::Accessor::Fast/;
-
-our $VERSION = "0.06";
+our $VERSION = "0.07";
 
 use Digest;
 use Digest::MoreFallbacks;
 
-use Perl6::Junction qw(any);
-
 use Carp qw/croak/;
 
 use Sub::Exporter;
+use Data::OptList;
 
-BEGIN {
-	our @DEFAULT_ACCESSORS = qw/
-		mode
-		authenticated_mode
-		encode
-		encoding
-		digest
-		cipher
-		mac
-		key
-		uri_encoding
-		printable_encoding
-		use_literal_key
-		tamper_proof_unencrypted
-	/;
+use namespace::clean -except => [qw(meta)];
 
-	__PACKAGE__->mk_accessors( map { "default_$_" } @DEFAULT_ACCESSORS );
+our %DEFAULT_ACCESSORS = (
+	mode                     => { isa => "Str" },
+	authenticated_mode       => { isa => "Str" },
+	encode                   => { isa => "Bool" },
+	encoding                 => { isa => "Str" },
+	printable_encoding       => { isa => "Str" },
+	alphanumerical_encoding  => { isa => "Str" },
+	uri_encoding             => { isa => "Str" },
+	digest                   => { isa => "Str" },
+	cipher                   => { isa => "Str" },
+	mac                      => { isa => "Str" },
+	key                      => { isa => "Str" },
+	uri_encoding             => { isa => "Str" },
+	printable_encoding       => { isa => "Str" },
+	use_literal_key          => { isa => "Bool" },
+	tamper_proof_unencrypted => { isa => "Bool" },
+	nonce                    => { isa => "Str", default => "" },
+);
 
-	__PACKAGE__->mk_accessors("disable_fallback");
+our @DEFAULT_ACCESSORS = keys %DEFAULT_ACCESSORS;
 
-	my %export_groups = (
-		'crypt' => [qw/
-			encrypt_string decrypt_string
-			authenticated_encrypt_string
-			tamper_proof thaw_tamper_proof
-			cipher_object
-		/],
-		digest => [qw/
-			digest_string verify_hash verify_digest
-			digest_object
-			mac_digest_string
-			verify_mac
-		/],
-		encoding => [qw/
-			encode_string decode_string
-			encode_string_hex decode_string_hex
-			encode_string_base64 decode_string_base64 encode_string_base64_wrapped
-			encode_string_base32 decode_string_base32
-			encode_string_uri_base64 decode_string_uri_base64
-			encode_string_uri decode_string_uri
-			encode_string_alphanumerical decode_string_alphanumerical
-			encode_string_printable decode_string_printable
-			encode_string_uri_escape decode_string_uri_escape
-		/],
-		params => [ "exported_instance", "disable_fallback", map { "default_$_" } @DEFAULT_ACCESSORS ],
-	);
+my %export_groups = (
+	'crypt' => [qw/
+		encrypt_string decrypt_string
+		authenticated_encrypt_string
+		tamper_proof thaw_tamper_proof
+		cipher_object
+	/],
+	digest => [qw/
+		digest_string verify_hash verify_digest
+		digest_object
+		mac_digest_string
+		verify_mac
+	/],
+	encoding => [qw/
+		encode_string decode_string
+		encode_string_hex decode_string_hex
+		encode_string_base64 decode_string_base64 encode_string_base64_wrapped
+		encode_string_base32 decode_string_base32
+		encode_string_uri_base64 decode_string_uri_base64
+		encode_string_uri decode_string_uri
+		encode_string_alphanumerical decode_string_alphanumerical
+		encode_string_printable decode_string_printable
+		encode_string_uri_escape decode_string_uri_escape
+	/],
+	params => [ "exported_instance", "disable_fallback", map { "default_$_" } @DEFAULT_ACCESSORS ],
+);
 
-	my %exports = map { $_ => \&__curry_instance } map { @$_ } values %export_groups;
+my %exports = map { $_ => \&__curry_instance } map { @$_ } values %export_groups;
 
-	Sub::Exporter->import( -setup => {
-		exports    => \%exports,
-		groups     => \%export_groups,
-		collectors => {
-			defaults => sub { 1 },
-		},
-	});
-}
+Sub::Exporter->import( -setup => {
+	exports    => \%exports,
+	groups     => \%export_groups,
+	collectors => {
+		defaults => sub { 1 },
+	},
+});
 
-our @KNOWN_AUTHENTICATING_MODES = qw(EAX OCB GCM CWC CCM),
+our @KNOWN_AUTHENTICATING_MODES = qw(EAX OCB GCM CWC CCM); # IACBC & IAPM will probably never be implemented
+
+our %KNOWN_AUTHENTICATING_MODES = map { $_ => 1 } @KNOWN_AUTHENTICATING_MODES;
 
 our %FALLBACK_LISTS = (
 	mode                    => [qw/CFB CBC Ctr OFB/],
@@ -85,7 +83,7 @@ our %FALLBACK_LISTS = (
 	block_mode              => [qw/CBC/],
 	authenticated_mode      => [qw/EAX GCM CCM/], # OCB/], OCB is patented
 	cipher                  => [qw/Rijndael Serpent Twofish RC6 Blowfish RC5/],
-# fa884d2582f6f05a5ffc6634115076027e7e91e1
+	#authenticated_cipher    => [qw/Phelix SOBER-128 Helix/], # not yet ready
 	digest                  => [qw/SHA-1 SHA-256 RIPEMD160 Whirlpool MD5 Haval256/],
 	mac                     => [qw/HMAC CMAC/],
 	encoding                => [qw/hex/],
@@ -126,6 +124,26 @@ foreach my $fallback ( keys %FALLBACK_LISTS ) {
 	*{ $list_method } = $list_method_sub;
 }
 
+foreach my $attr ( @DEFAULT_ACCESSORS ) {
+	has "default_$attr" => (
+		is => "rw",
+		predicate => "has_default_$attr",
+		clearer   => "clear_default_$attr",
+		( __PACKAGE__->can("fallback_$attr") ? (
+			lazy_build => 1,
+			builder => "fallback_$attr",
+		) : () ),
+		%{ $DEFAULT_ACCESSORS{$attr} },
+	);
+}
+
+has disable_fallback => (
+	isa => "Bool",
+	is  => "rw",
+);
+
+__PACKAGE__->meta->make_immutable;
+
 {
 	my %fallback_caches;
 
@@ -154,10 +172,12 @@ sub _try_cipher_fallback {
 sub _try_digest_fallback {
 	my ( $self, $name ) = @_;
 
-	my $e = do {
+	my $e;
+
+	{
 		local $@;
 		eval { $self->digest_object( digest => $name ) };
-		$@;
+		$e = $@;
 	};
 
 	return 1 if !$e;
@@ -181,21 +201,22 @@ sub _try_loading_module {
 
 	(my $file = "${name}.pm") =~ s{::}{/}g;
 
-	my $e = do {
+	my ( $r, $e );
+	{
 		local $@;
-		eval { require $file }; # yes it's portable
-		$@;
+		$r = eval { require $file }; # yes it's portable
+		$e = $@;
 	};
 
-	return 1 if !$e;
+	return $r if $r;
 	die $e if $e !~ /^Can't locate \Q$file\E in \@INC/;
-	return;
+	return $r;
 }
 
 {
 	my %encoding_module = (
 		base64     => "MIME::Base64",
-		uri_base64 => "MIME::Base64",
+		uri_base64 => "MIME::Base64::URLSafe",
 		base32     => "MIME::Base32",
 		uri_escape => "URI::Escape",
 	);
@@ -254,16 +275,11 @@ sub _process_param {
 
 	my $default = "default_$param";
 
-	if ( $self->can($default) and defined( my $value = $self->$default ) ) {
-		return $value;
+	if ( $self->can($default) ) {
+		return $self->$default;
 	}
 
-	my $fallback = "fallback_$param";
-	if ( $self->can($fallback) ) {
-		return $self->$fallback;
-	} else {
-		croak "No default value for required parameter '$param'";
-	}
+	croak "No default value for required parameter '$param'";
 }
 
 sub cipher_object {
@@ -281,7 +297,7 @@ sub cipher_object {
 sub cipher_object_eax {
 	my ( $self, %params ) = _args @_;
 
-	$self->_process_params( \%params, qw/cipher/ );
+	$self->_process_params( \%params, qw/cipher nonce/ );
 
 	require Crypt::EAX;
 
@@ -289,6 +305,7 @@ sub cipher_object_eax {
 		%params,
 		cipher => "Crypt::$params{cipher}", # FIXME take a ref, but Crypt::CFB will barf
 		key    => $self->process_key(%params),
+		nonce  => $params{nonce},
 	);
 }
 
@@ -345,26 +362,29 @@ sub _cipher_object_baurem {
 	$class->new( $self->process_key(%params), join("::", $prefix, $params{cipher}) );
 }
 
-
-sub cipher_object_stream {
+use tt;
+[% FOR mode IN ["stream", "block", "authenticated"] %]
+sub cipher_object_[% mode %] {
 	my ( $self, @args ) = _args @_;
-	my $mode = $self->_process_param("stream_mode");
+	my $mode = $self->_process_param("[% mode %]_mode");
 	$self->cipher_object( @args, mode => $mode );
 }
+[% END %]
+no tt;
 
-sub cipher_object_block {
-	my ( $self, @args ) = _args @_;
-	my $mode = $self->_process_param("block_mode");
-	$self->cipher_object( @args, mode => $mode );
+sub process_nonce {
+	my ( $self, %params ) = _args @_, 'nonce';
+
+	my $nonce = $self->_process_params( \%params, 'nonce' );
+
+	if ( length($nonce) ) {
+		return $nonce;
+	} else {
+		require Data::GUID;
+		Data::GUID->new->as_binary;
+	}
 }
 
-sub cipher_object_authenticated {
-	my ( $self, @args ) = _args @_;
-	my $mode = $self->_process_param("authenticated_mode");
-	$self->cipher_object( @args, mode => $mode );
-}
-
-
 sub process_key {
 	my ( $self, %params ) = _args @_, "key";
 
@@ -379,8 +399,12 @@ sub process_key {
 			my $cipher = $params{cipher};
 
 			my $class = "Crypt::$cipher";
-			my $size_method = $class->can("keysize") || $class->can("blocksize");
-			$size = $class->$size_method;
+
+			$self->_try_loading_module($class);
+
+			if ( my $size_method = $class->can("keysize") || $class->can("blocksize") ) {
+				$size = $class->$size_method;
+			}
 
 			$size ||= $cipher eq "Blowfish" ? 56 : 32;
 		}
@@ -408,7 +432,7 @@ sub digest_object {
 }
 
 {
-# c423c07872e6e8002ee9c826c602c9e5e9ec705a
+	# this is a hack that gives to Digest::HMAC something that responds to ->new
 
 	package
 	Crypt::Util::HMACDigestFactory;
@@ -455,12 +479,30 @@ sub mac_object_hmac {
 	Digest::HMAC->new(
 		$key,
 		$digest_factory,
-# 8f6ca5a32f36f2311a153d671093109b46db764e
+		# FIXME hmac_block_size param?
 	);
 }
 
+sub mac_object_cmac {
+	my ( $self, %params ) = _args @_;
 
-sub encrypt_string {
+	my ( $key, $cipher );
+
+	if ( ref $params{cipher} ) {
+		$cipher = $params{cipher};
+	} else {
+		$self->_process_params( \%params, qw(cipher) );
+		$cipher = "Crypt::" . $params{cipher};
+		$key    = $self->process_key(%params);
+	}
+
+	require Digest::CMAC;
+	Digest::CMAC->new( $key, $cipher );
+}
+
+use tt;
+[% FOR f IN ["en", "de"] %]
+sub [% f %]crypt_string {
 	my ( $self, %params ) = _args @_, "string";
 
 	my $string = delete $params{string};
@@ -468,50 +510,22 @@ sub encrypt_string {
 
 	my $c = $self->cipher_object( %params );
 
-	
+	[% IF f == "en" %]
 	$self->maybe_encode( $c->encrypt($string), \%params );
-	
-}
-
-sub maybe_encode {
-	my ( $self, $string, $params ) = @_;
-
-	my $should_encode = exists $params->{encode}
-		? $params->{encode}
-		: exists $params->{encoding} || $self->default_encode;
-
-	if ( $should_encode ) {
-		return $self->encode_string(
-			%$params,
-			string   => $string,
-		);
-	} else {
-		return $string;
-	}
-}
-
-sub decrypt_string {
-	my ( $self, %params ) = _args @_, "string";
-
-	my $string = delete $params{string};
-	croak "You must provide the 'string' parameter" unless defined $string;
-
-	my $c = $self->cipher_object( %params );
-
-	
+	[% ELSE %]
 	$c->decrypt( $self->maybe_decode($string, \%params ) );
-	
+	[% END %]
 }
 
-sub maybe_decode {
+sub maybe_[% f %]code {
 	my ( $self, $string, $params ) = @_;
 
-	my $should_encode = exists $params->{decode}
-		? $params->{decode}
+	my $should_encode = exists $params->{[% f %]code}
+		? $params->{[% f %]code}
 		: exists $params->{encoding} || $self->default_encode;
 
 	if ( $should_encode ) {
-		return $self->decode_string(
+		return $self->[% f %]code_string(
 			%$params,
 			string   => $string,
 		);
@@ -519,7 +533,8 @@ sub maybe_decode {
 		return $string;
 	}
 }
-
+[% END %]
+no tt;
 
 sub _digest_string_with_object {
 	my ( $self, $object, %params ) = @_;
@@ -618,9 +633,10 @@ sub verify_mac {
 sub tamper_proof {
 	my ( $self, %params ) = _args @_, "data";
 
-	my $packed = $self->pack_data( %params );
+	$params{string} = $self->pack_data( %params );
+	#$params{header} = $self->pack_data( %params, data => $params{header} ) if exists $params{header}; # FIXME this is not yet finished
 
-	$self->tamper_proof_string( %params, string => $packed );
+	$self->tamper_proof_string( %params );
 }
 
 sub freeze_data {
@@ -675,20 +691,20 @@ sub tamper_proof_string {
 sub _authenticated_mode {
 	my ( $self, $params ) = @_;
 
-# d8ca7f81e88c105a34bbdfeca3f345ae6b895ab4
+	# trust explicit param
 	if ( exists $params->{authenticated_mode} ) {
 		$params->{mode} = delete $params->{authenticated_mode};
 		return 1;
 	}
 
-# 0f84e32ca8a994cc47b73f87ebe2255970458187
+	# check if the explicit param is authenticated
 	if ( exists $params->{mode} ) {
-# 90064c4d2aeb8499c4f9c733a18ef416b8d667bf
+		# allow overriding
 		if ( exists $params->{mode_is_authenticated} ) {
 			return $params->{mode_is_authenticated};
 		}
 
-		if ( any( map { lc } @KNOWN_AUTHENTICATING_MODES ) eq lc($params->{mode}) ) {
+		if ( $KNOWN_AUTHENTICATING_MODES{uc($params->{mode})} ) {
 			return 1;
 		} else {
 			return;
@@ -767,9 +783,18 @@ sub _unpack_version_flags_and_string {
 sub authenticated_encrypt_string {
 	my ( $self, %params ) = _args @_, "string";
 
-# 0d10af4c2d455ed7ab4f4524fbbd8fb8a81e3c6b
+	# FIXME some ciphers are authenticated, but none are implemented in perl yet
 	if ( $self->_authenticated_mode(\%params) ) {
-		return $self->encrypt_string( %params );
+		$self->_process_params( \%params, qw(nonce) );
+
+		# generate a nonce unless one is explicitly provided
+		my $nonce = $self->process_nonce(%params); # FIMXE limit to 64k?
+
+		# FIXME safely encode an arbitrary header as well
+		#my $header = $params{header};
+		#$header = '' unless defined $header;
+
+		return pack("n/a a*", $nonce, $self->encrypt_string( %params, nonce => $nonce ) );
 	} else {
 		croak "To use encrypted tamper resistent strings an authenticated encryption mode such as EAX must be selected";
 	}
@@ -779,9 +804,15 @@ sub authenticated_decrypt_string {
 	my ( $self, %params ) = _args @_, "string";
 
 	if ( $self->_authenticated_mode(\%params) ) {
+		$self->_process_params( \%params, qw(string) );
+
+		my ( $nonce, $string ) = unpack("n/a a*", $params{string});
+
 		return $self->decrypt_string(
 			fatal => 1,
 			%params,
+			nonce  => $nonce,
+			string => $string,
 		);
 	} else {
 		croak "To use encrypted tamper resistent strings an authenticated encryption mode such as EAX must be selected";
@@ -855,8 +886,9 @@ sub _packed_string_version_check {
 		unless $version == $PACK_FORMAT_VERSION;
 }
 
-
-sub encode_string {
+use tt;
+[% FOR f IN ["en","de"] %]
+sub [% f %]code_string {
 	my ( $self, %params ) = _args @_, "string";
 
 	my $string = delete $params{string};
@@ -868,30 +900,13 @@ sub encode_string {
 
 	my $encoding = delete $params{encoding};
 	croak "Encoding method must be an encoding name" unless $encoding;
-	my $method = "encode_string_$encoding";
+	my $method = "[% f %]code_string_$encoding";
 	croak "Encoding method $encoding is not supported" unless $self->can($method);
 
 	$self->$method($string);
 }
-
-sub decode_string {
-	my ( $self, %params ) = _args @_, "string";
-
-	my $string = delete $params{string};
-	croak "You must provide the 'string' parameter" unless defined $string;
-
-	$self->_process_params( \%params, qw/
-		encoding
-	/);
-
-	my $encoding = delete $params{encoding};
-	croak "Encoding method must be an encoding name" unless $encoding;
-	my $method = "decode_string_$encoding";
-	croak "Encoding method $encoding is not supported" unless $self->can($method);
-
-	$self->$method($string);
-}
-
+[% END %]
+no tt;
 
 sub encode_string_hex {
 	my ( $self, $string ) = @_;
@@ -921,20 +936,17 @@ sub decode_string_base64 {
 	MIME::Base64::decode_base64($base64);
 }
 
-# 246d37740a5c4c6a218f83be4355351b9fb8bdbd
+# http://www.dev411.com/blog/2006/10/02/encoding-hashed-uids-base64-vs-hex-vs-base32
 sub encode_string_uri_base64 {
 	my ( $self, $string ) = @_;
-	my $encoded = $self->encode_string_base64($string);
-	$encoded =~ tr{+/}{*-};
-	$encoded =~ s/=+$//;
-	return $encoded;
+	require MIME::Base64::URLSafe;
+	MIME::Base64::URLSafe::encode($string);
 }
 
 sub decode_string_uri_base64 {
 	my ( $self, $base64 ) = @_;
-	$base64 =~ tr{*-}{+/};
-	$base64 .= "=" x abs( - length($base64) % 4 );
-	$self->decode_string_base64($base64);
+	require MIME::Base64::URLSafe;
+	MIME::Base64::URLSafe::decode($base64);
 }
 
 sub encode_string_base32 {
@@ -961,49 +973,17 @@ sub decode_string_uri_escape {
 	URI::Escape::uri_unescape($uri_escaped);
 }
 
-
-
-sub encode_string_uri {
+use tt;
+[% FOR symbolic_encoding IN ["uri", "alphanumerical", "printable"] %]
+[% FOR f IN ["en", "de"] %]
+sub [% f %]code_string_[% symbolic_encoding %] {
 	my ( $self, $string ) = @_;
-	my $encoding = $self->_process_param("uri_encoding");
-	$self->encode_string( string => $string, encoding => $encoding );
+	my $encoding = $self->_process_param("[% symbolic_encoding %]_encoding");
+	$self->[% f %]code_string( string => $string, encoding => $encoding );
 }
-
-sub decode_string_uri {
-	my ( $self, $string ) = @_;
-	my $encoding = $self->_process_param("uri_encoding");
-	$self->decode_string( string => $string, encoding => $encoding );
-}
-
-
-
-sub encode_string_alphanumerical {
-	my ( $self, $string ) = @_;
-	my $encoding = $self->_process_param("alphanumerical_encoding");
-	$self->encode_string( string => $string, encoding => $encoding );
-}
-
-sub decode_string_alphanumerical {
-	my ( $self, $string ) = @_;
-	my $encoding = $self->_process_param("alphanumerical_encoding");
-	$self->decode_string( string => $string, encoding => $encoding );
-}
-
-
-
-sub encode_string_printable {
-	my ( $self, $string ) = @_;
-	my $encoding = $self->_process_param("printable_encoding");
-	$self->encode_string( string => $string, encoding => $encoding );
-}
-
-sub decode_string_printable {
-	my ( $self, $string ) = @_;
-	my $encoding = $self->_process_param("printable_encoding");
-	$self->decode_string( string => $string, encoding => $encoding );
-}
-
-
+[% END %]
+[% END %]
+no tt;
 
 sub exported_instance {
 	my $self = shift;
@@ -1068,6 +1048,15 @@ Crypt::Util - A lightweight Crypt/Digest convenience API
 This module provides an easy, intuitive and forgiving API for wielding
 crypto-fu.
 
+The API is designed as a cascade, with rich features built using simpler ones.
+this means that the option processing is uniform throughout, and the behaviors
+are generally predictable.
+
+Note that L<Crypt::Util> doesn't do any crypto on its own, but delegates the
+actual work to the various other crypto modules on the CPAN. L<Crypt::Util>
+merely wraps these modules, providing uniform parameters, and building on top
+of their polymorphism with higher level features.
+
 =head2 Priorities
 
 =over 4
@@ -1145,9 +1134,13 @@ handling default values.
 
 =item thaw_tamper_proof( [ $string ], %params )
 
-=item tamper_proof_string $string, %params
+=item tamper_proof_string( $string, %params )
 
-=item thaw_tamper_proof_string $string, %params
+=item thaw_tamper_proof_string( $string, %params )
+
+=item aead_tamper_proof_string( [ $string ], %params )
+
+=item mac_tamper_proof_string( [ $string ], %params )
 
 The C<tamper_proof> method is in an intermittent state, in that the C<data>
 parameter's API is not completely finalized.
@@ -1158,8 +1151,9 @@ same in future versions as well.
 See L</TODO> for more information about the data types that will be supported
 in the future.
 
-When thawing, the C<verify_digest> or C<verify_mac> methods will be used, with
-C<fatal> defaulting to on unless explicitly disabled in the parameters.
+When thawing, the C<authenticated_decrypt_string> or C<verify_mac> methods will
+be used, with C<fatal> defaulting to on unless explicitly disabled in the
+parameters.
 
 =over 4
 
@@ -1171,8 +1165,8 @@ By default this parameter is true, unless C<default_tamper_proof_unencrypted()>,
 has been enabled.
 
 A true value implies that all the parameters
-which are available to C<encrypt_string()> are also available.  If a
-negative value is specified, MAC mode is used, and the additional
+which are available to C<authenticated_encrypt_string()> are also available.
+If a negative value is specified, MAC mode is used, and the additional
 parameters of C<mac_digest_string()> may also be specified to this method.
 
 =item * data
@@ -1180,6 +1174,8 @@ parameters of C<mac_digest_string()> may also be specified to this method.
 The data to encrypt. If this is a reference L<Storable> will be used to
 serialize the data.
 
+See C<pack_data> for details.
+
 =back
 
 If the string is encrypted then all the parameters of C<encrypt_string> and
@@ -1192,9 +1188,16 @@ are also available.
 
 =item decrypt_string( [ $string ], %params )
 
+=item authenticated_encrypt_string( [ $string ], %params )
+
+=item authenticated_decrypt_string( [ $string ], %params )
+
 All of the parameters which may be supplied to C<process_key()>,
 C<cipher_object> and C<maybe_encode> are also available to these methods.
 
+The C<authenticated> variants ensure that an authenticated encryption mode
+(such as EAX) is used.
+
 The following parameters may be used:
 
 =over 4
@@ -1204,9 +1207,21 @@ The following parameters may be used:
 The string to be en/decrypted can either be supplied first, creating an odd
 number of arguments, or as a named parameter.
 
+=item * nonce
+
+The cryptographic nonce to use. Only necessary for encryption, will be packed
+in the string as part of the message if applicable.
+
+=item * header
+
+Not yet supported.
+
+In the future this will include a header for AEAD (the "associated data" bit of
+AEAD).
+
 =back
 
-=item process_key( $key, %params )
+=item process_key( [ $key ], %params )
 
 The following arguments may be specified:
 
@@ -1228,6 +1243,26 @@ Used to determine the key size.
 
 =back
 
+=item process_nonce( [ $nonce ], %params )
+
+If a nonce is explicitly specified this method returns that, and otherwise uses
+L<Data::GUID> to generate a unique binary string for use as a nonce/IV.
+
+=item pack_data( [ $data ], %params )
+
+=item unpack_data( [ $data ], %params )
+
+Uses L<Storable> and C<pack> to create a string out of data.
+
+L<MooseX::Storage> support will be added in the future.
+
+The format itself is versioned in order to facilitate future proofing and
+backwards compatibility.
+
+Note that it is not safe to call C<unpack_data> on an untrusted string, use
+C<thaw_tamper_proof> instead (it will authenticate the data and only then
+perform the potentially unsafe routines).
+
 =item cipher_object( %params )
 
 Available parameters are:
@@ -1236,17 +1271,44 @@ Available parameters are:
 
 =item * cipher
 
-The cipher algorithm to use.
+The cipher algorithm to use, e.g. C<Rijndael>, C<Twofish> etc.
 
 =item * mode
 
-The mode of operation (C<cbc>, C<cfb>).
+The mode of operation. This can be real (C<cbc>, C<cfb>, C<ctr>, C<ofb>,
+C<eax>) or symbolic (C<authenticated>, C<block>, C<stream>).
 
 See L<http://en.wikipedia.org/wiki/Block_cipher_modes_of_operation> for an
 explanation of this.
 
 =back
 
+=item cipher_object_eax( %params )
+
+Used by C<cipher_object> but accepts additional parameters:
+
+=over 4
+
+=item * nonce
+
+The nonce is a value that should be unique in order to protect against replay
+attacks. It also ensures that the same plain text with the same key will
+produce different ciphertexts.
+
+The nonce is not included in the output ciphertext. See
+C<authenticated_encrypt_string> for a convenience method that does include the
+nonce.
+
+=item * header
+
+This is additional data to authenticate but not encrypt.
+
+See L<Crypt::EAX> for more details.
+
+The header will not be included in the output ciphertext.
+
+=back
+
 =item digest_string( [ $string ], %params )
 
 Delegates to C<digest_object>. All parameters which can be used by
@@ -1352,13 +1414,13 @@ failures will return undef.
 
 =back
 
-=item mac_object
+=item mac_object( %params )
 
 =over 4
 
 =item * mac
 
-The MAC algorithm to use. Currently only C<hmac> is supported.
+The MAC algorithm to use. Currently C<hmac> and C<cmac> are supported.
 
 =back
 
@@ -1550,6 +1612,13 @@ Whether or not to encode by default (applies to digests and encryptions).
 
 The key to use. Useful for when you are repeatedly encrypting.
 
+=item * nonce
+
+The nonce/IV to use for cipher modes that require it.
+
+Defaults to the empty string, but note that some methods will generate a nonce
+for you (e.g. C<authenticated_encrypt_string>) if none was provided.
+
 =item * use_literal_key
 
 Whether or not to not hash the key by default. See C<process_key>.
@@ -1565,15 +1634,10 @@ Whether or not tamper resistent strings are by default unencrypted (just MAC).
 You may safely subclass and override C<default_PARAMETER> and
 C<fallback_PARAMETER_list> to provide values from configurations.
 
-Overriding the C<fallback_PARAMETER> method is also "allowed" but not
-reccomended.
-
 =back
 
 =head1 TODO
 
-Features which are currently missing but are scheduled for 0.02 or 0.03:
-
 =over 4
 
 =item *
@@ -1582,20 +1646,16 @@ Crypt::SaltedHash support
 
 =item *
 
-=item *
-
-CMAC, EMAC (maybe, the modules are not OO and require refactoring) message
-authentication modes.
-
-=item *
-
-EAX/OCB encryption mode (with implicit authentication)
+EMAC (maybe, the modules are not OO and require refactoring) message
+authentication mode
 
 =item *
 
 Bruce Schneier Fact Database
 L<http://geekz.co.uk/lovesraymond/archive/bruce-schneier-facts>.
 
+L<WWW::SchneierFacts>
+
 =item *
 
 Entropy fetching (get N weak/strong bytes, etc) from e.g. OpenSSL bindings,
@@ -1608,6 +1668,14 @@ data/string handling for all methods, not just tamper_proof).
 
 Streams should also be able to used via a simple push api.
 
+=item *
+
+IV/nonce/salt support for the various cipher modes, not just EAX (CBC, CCM, GCM, etc)
+
+=item *
+
+L<Crypt::Rijndael> can do its own cipher modes
+
 =head1 SEE ALSO
 
 L<Digest>, L<Crypt::CBC>, L<Crypt::CFB>,
